@@ -1,6 +1,6 @@
 # - coding: utf-8 -
 
-# Copyright (C) 2007-2009 Toms Bauģis <toms.baugis at gmail.com>
+# Copyright (C) 2007-2010 Toms Bauģis <toms.baugis at gmail.com>
 # Copyright (C) 2007-2009 Patryk Zawadzki <patrys at pld-linux.org>
 # Copyright (C) 2008 Pēteris Caune <cuu508 at gmail.com>
 
@@ -29,13 +29,11 @@ import gtk
 import gnomeapplet
 import gobject
 import dbus, dbus.service, dbus.mainloop.glib
+import locale
 
-import eds
 from configuration import conf, runtime, dialogs
 
 import stuff
-import keybinder
-from hamsterdbus import HAMSTER_URI, HamsterDbusController
 
 # controllers for other windows
 import widgets
@@ -199,7 +197,7 @@ class HamsterApplet(object):
         self._gui = stuff.load_ui_file("applet.ui")
         self.window = self._gui.get_object('hamster-window')
         # on close don't destroy the popup, just hide it instead
-        self.window.connect("delete_event", lambda *args: self.__show_toggle(None, False))
+        self.window.connect("delete_event", lambda *args: self.__show_toggle(False))
         self.window.connect("window-state-event", self.on_window_state_changed)
 
         self.new_name = widgets.ActivityEntry()
@@ -226,12 +224,9 @@ class HamsterApplet(object):
         # DBus Setup
         try:
             dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
-            name = dbus.service.BusName(HAMSTER_URI, dbus.SessionBus())
-            self.dbusController = HamsterDbusController(bus_name = name)
-
             # Set up connection to the screensaver
-            self.dbusIdleListener = idle.DbusIdleListener(runtime.dispatcher)
-            runtime.dispatcher.add_handler('active_changed', self.on_idle_changed)
+            self.dbusIdleListener = idle.DbusIdleListener()
+            self.dbusIdleListener.connect('idle-changed', self.on_idle_changed)
 
         except dbus.DBusException, e:
             logging.error("Can't init dbus: %s" % e)
@@ -242,25 +237,19 @@ class HamsterApplet(object):
         self.notify_interval = conf.get("notify_interval")
         self.workspace_tracking = conf.get("workspace_tracking")
 
-        self.hotkey = conf.get("keybinding")
-        self.bind_hotkey()
-
-        runtime.dispatcher.add_handler('conf_changed', self.on_conf_changed)
+        conf.connect('conf-changed', self.on_conf_changed)
 
         # Load today's data, activities and set label
         self.last_activity = None
         self.load_day()
         self.update_label()
 
-        # Hamster DBusController current fact initialising
-        self.__update_fact()
 
         # refresh hamster every 60 seconds to update duration
         gobject.timeout_add_seconds(60, self.refresh_hamster)
-
-        runtime.dispatcher.add_handler('panel_visible', self.__show_toggle)
-        runtime.dispatcher.add_handler('activity_updated', self.after_activity_update)
-        runtime.dispatcher.add_handler('day_updated', self.after_fact_update)
+        runtime.storage.connect('activities-changed', self.after_activity_update)
+        runtime.storage.connect('facts-changed', self.after_fact_update)
+        runtime.storage.connect('toggle-called', self.on_toggle_called)
 
         self.screen = None
         if self.workspace_tracking:
@@ -276,15 +265,6 @@ class HamsterApplet(object):
 
         self.prev_size = None
 
-
-    def bind_hotkey(self):
-        try:
-            print 'Binding shortcut %s to popup hamster' % self.hotkey
-            keybinder.bind(self.hotkey, self.on_keybinding_activated)
-        except KeyError:
-            pass # don't care
-
-
     def init_workspace_tracking(self):
         if not wnck: # can't track if we don't have the trackable
             return
@@ -385,7 +365,7 @@ class HamsterApplet(object):
             total_strings = []
             for category in by_category:
                 # listing of today's categories and time spent in them
-                duration = "%.1f" % (by_category[category] / 60.0)
+                duration = locale.format("%.1f", (by_category[category] / 60.0))
                 total_strings.append(_("%(category)s: %(duration)s") % \
                         ({'category': category,
                           #duration in main drop-down per category in hours
@@ -410,11 +390,10 @@ class HamsterApplet(object):
             delta = dt.datetime.now() - activity['start_time']
             duration = delta.seconds /  60
 
-            self.get_widget("last_activity_name").set_text(activity['name'])
             if activity['category'] != _("Unsorted"):
-                self.get_widget("last_activity_category") \
-                    .set_text(" - %s" % activity['category'])
-            self.get_widget("last_activity_category").show()
+                self.get_widget("last_activity_name").set_text("%s - %s" % (activity['name'], activity['category']))
+            else:
+                self.get_widget("last_activity_name").set_text(activity['name'])
 
 
             self._gui.get_object("more_info_button").hide()
@@ -429,7 +408,6 @@ class HamsterApplet(object):
             self.get_widget("start_tracking").show()
 
             self.get_widget("last_activity_name").set_text(_("No activity"))
-            self.get_widget("last_activity_category").hide()
 
             self.get_widget("activity_info_box").hide()
             self._gui.get_object("more_info_button").show()
@@ -441,18 +419,7 @@ class HamsterApplet(object):
         fact = self.treeview.get_selected_fact()
         runtime.storage.remove_fact(fact["id"])
 
-    def __update_fact(self):
-        """dbus controller current fact updating"""
-        last_activity_id = 0
-
-        if not self.last_activity:
-            self.dbusController.TrackingStopped()
-        else:
-            last_activity_id = self.last_activity['id']
-
-        self.dbusController.FactUpdated(last_activity_id)
-
-    def __show_toggle(self, event, is_active):
+    def __show_toggle(self, is_active):
         """main window display and positioning"""
         self.button.set_active(is_active)
 
@@ -460,9 +427,6 @@ class HamsterApplet(object):
             self.window.hide()
             return True
 
-        self.load_day() # reload day each time before showing to avoid outdated last activity
-        self.update_label() #update also label, otherwise we can get 1 minute difference in durations (due to timers refreshed once a minute)
-
         self.position_popup()
 
 
@@ -513,6 +477,9 @@ class HamsterApplet(object):
         self.window.present()
         self.new_name.grab_focus()
 
+        self.load_day() # reload day each time before showing to avoid outdated last activity
+        self.update_label() #update also label, otherwise we can get 1 minute difference in durations (due to timers refreshed once a minute)
+
 
     """events"""
     def on_window_state_changed(self, window, event):
@@ -523,7 +490,7 @@ class HamsterApplet(object):
 
 
     def on_toggle(self, widget):
-        self.__show_toggle(None, self.button.get_active())
+        self.__show_toggle(self.button.get_active())
 
 
     def on_todays_keys(self, tree, event):
@@ -545,7 +512,7 @@ class HamsterApplet(object):
                                      ", ".join(fact["tags"]),
                                      category_name = fact["category"],
                                      description = fact["description"])
-            runtime.dispatcher.dispatch('panel_visible', False)
+            self.__show_toggle(False)
 
 
     def on_windows_keys(self, tree, event_key):
@@ -554,13 +521,13 @@ class HamsterApplet(object):
               and event_key.state & gtk.gdk.CONTROL_MASK)):
             if self.new_name.popup.get_property("visible") == False \
                and self.new_tags.popup.get_property("visible") == False:
-                runtime.dispatcher.dispatch('panel_visible', False)
+                self.__show_toggle(False)
                 return True
         return False
 
     """button events"""
     def on_overview(self, menu_item):
-        runtime.dispatcher.dispatch('panel_visible', False)
+        self.__show_toggle(False)
         dialogs.overview.show(self.applet)
 
     def show_overview(self, menu_item, verb):
@@ -570,23 +537,22 @@ class HamsterApplet(object):
         dialogs.edit.show(self.applet)
 
     def on_about (self, component, verb):
-        dialogs.about.show()
+        dialogs.about.show(self.window)
 
     def show_preferences(self, menu_item, verb):
-        runtime.dispatcher.dispatch('panel_visible', False)
+        self.__show_toggle(False)
         dialogs.prefs.show(self.applet)
 
 
     """signals"""
-    def after_activity_update(self, widget, renames):
+    def after_activity_update(self, widget):
         self.new_name.refresh_activities()
         self.load_day()
         self.update_label()
 
-    def after_fact_update(self, event, date):
+    def after_fact_update(self, event):
         self.load_day()
         self.update_label()
-        self.__update_fact()
 
     def on_idle_changed(self, event, state):
         # state values: 0 = active, 1 = idle
@@ -598,8 +564,7 @@ class HamsterApplet(object):
         elif self.timeout_enabled and self.last_activity and \
              self.last_activity['end_time'] is None:
 
-            runtime.storage.touch_fact(self.last_activity,
-                                       end_time = self.dbusIdleListener.getIdleFrom())
+            runtime.storage.stop_tracking(self.dbusIdleListener.getIdleFrom())
 
     def on_workspace_changed(self, screen, previous_workspace):
         if not previous_workspace:
@@ -633,11 +598,11 @@ class HamsterApplet(object):
             if parsed_activity:
                 category_id = None
                 if parsed_activity.category_name:
-                    category_id = runtime.storage.get_category_by_name(parsed_activity.category_name)
+                    category_id = runtime.storage.get_category_id(parsed_activity.category_name)
 
                 activity = runtime.storage.get_activity_by_name(parsed_activity.activity_name,
                                                                 category_id,
-                                                                ressurect = False)
+                                                                resurrect = False)
                 if activity:
                     # we need dict below
                     activity = dict(name = activity['name'],
@@ -675,14 +640,10 @@ class HamsterApplet(object):
                                      "hamster-applet")
             self.notification.show()
 
-    """global shortcuts"""
-    def on_keybinding_activated(self):
-        self.__show_toggle(None, not self.button.get_active())
-
-
-    def on_conf_changed(self, event, data):
-        key, value = data
+    def on_toggle_called(self, client):
+        self.__show_toggle(not self.button.get_active())
 
+    def on_conf_changed(self, event, key, value):
         if key == "enable_timeout":
             self.timeout_enabled = value
         elif key == "notify_on_idle":
@@ -700,34 +661,26 @@ class HamsterApplet(object):
                 if self.screen:
                     self.screen.disconnect(self.screen.workspace_handler)
                     self.screen = None
-        elif key == "keybinding":
-            try:
-                keybinder.unbind(self.hotkey)
-            except KeyError:
-                pass # don't care
-
-            self.hotkey = value
-            self.bind_hotkey()
-
-
 
     def on_activity_text_changed(self, widget):
         self.get_widget("switch_activity").set_sensitive(widget.get_text() != "")
 
     def on_switch_activity_clicked(self, widget):
-        if not self.new_name.get_text():
+        activity_name, temporary = self.new_name.get_value()
+        if not activity_name:
             return False
 
         runtime.storage.add_fact(self.new_name.get_text().decode("utf8", "replace"),
-                                 self.new_tags.get_text().decode("utf8", "replace"))
+                                 self.new_tags.get_text().decode("utf8", "replace"),
+                                 temporary = temporary)
         self.new_name.set_text("")
         self.new_tags.set_text("")
-        runtime.dispatcher.dispatch('panel_visible', False)
+        self.__show_toggle(False)
 
     def on_stop_tracking_clicked(self, widget):
-        runtime.storage.touch_fact(self.last_activity)
+        runtime.storage.stop_tracking()
         self.last_activity = None
-        runtime.dispatcher.dispatch('panel_visible', False)
+        self.__show_toggle(False)
 
     def on_window_size_request(self, window, event):
         box = self.window.get_allocation()
