@@ -38,62 +38,72 @@ import storage
 import stuff
 from shutil import copy as copyfile
 import datetime as dt
-import gettext
+import gio
 
 import itertools
 
-DB_FILE = 'hamster.db'
-
 class Storage(storage.Storage):
     con = None # Connection will be created on demand
-    def __setup(self):
+    def __init__(self, loop):
         """
         Delayed setup so we don't do everything at the same time
         """
-        if self.__setup.im_func.complete:
-            return
+        storage.Storage.__init__(self, loop)
 
         self.__con = None
         self.__cur = None
+        self.__last_etag = None
 
         from configuration import runtime
 
-        db_file = runtime.database_path
-        db_path, _ = os.path.split(os.path.realpath(db_file))
-
-        if not os.path.exists(db_path):
-            try:
-                os.makedirs(db_path, 0744)
-            except Exception, msg:
-                logging.error("could not create user dir (%s): %s" % (db_path, msg))
+        self.db_path = os.path.join(runtime.home_data_dir, "hamster.db")
 
         data_dir = runtime.data_dir
 
         #check if db is here
-        if not os.path.exists(db_file):
-            logging.info("Database not found in %s - installing default from %s!" % (db_file, data_dir))
-            copyfile(os.path.join(data_dir, DB_FILE), db_file)
+        if not os.path.exists(self.db_path):
+            logging.info("Database not found in %s - installing default from %s!" % (self.db_path, data_dir))
+            copyfile(os.path.join(data_dir, 'hamster.db'), self.db_path)
 
             #change also permissions - sometimes they are 444
             try:
-                os.chmod(db_file, 0664)
+                os.chmod(self.db_path, 0664)
             except Exception, msg:
-                logging.error("Could not change mode on %s!" % (db_file))
-        self.__setup.im_func.complete = True
+                logging.error("Could not change mode on %s!" % (self.db_path))
+
+
+        # add file monitoring so the app does not have to be restarted
+        # when db file is rewritten
+        def on_db_file_change(monitor, gio_file, event_uri, event):
+            if event == gio.FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
+                if gio_file.query_info(gio.FILE_ATTRIBUTE_ETAG_VALUE).get_etag() == self.__last_etag:
+                    # ours
+                    return
+
+                print "DB file has been modified externally. Calling all stations"
+                self.dispatch_overwrite()
+
+
+
+        self.__database_file = gio.File(self.db_path)
+        self.__db_monitor = self.__database_file.monitor_file()
+        self.__db_monitor.connect("changed", on_db_file_change)
+
         self.run_fixtures()
 
 
 
-    __setup.complete = False
+    def register_modification(self):
+        # db.execute calls this so we know that we were the ones
+        # that modified the DB and no extra refesh is not needed
+        self.__last_etag = self.__database_file.query_info(gio.FILE_ATTRIBUTE_ETAG_VALUE).get_etag()
 
     #tags, here we come!
-    def __get_tags(self, autocomplete = None):
-        query = "select * from tags"
-        if autocomplete:
-            query += " where autocomplete='true'"
-
-        query += " order by name"
-        return self.fetchall(query)
+    def __get_tags(self, only_autocomplete = False):
+        if only_autocomplete:
+            return self.fetchall("select * from tags where autocomplete != 'false' order by name")
+        else:
+            return self.fetchall("select * from tags order by name")
 
     def __get_tag_ids(self, tags):
         """look up tags by their name. create if not found"""
@@ -103,7 +113,7 @@ class Storage(storage.Storage):
 
         changes = False
 
-        # check if any of tags needs ressurection
+        # check if any of tags needs resurrection
         set_complete = [str(tag["id"]) for tag in db_tags if tag["autocomplete"] == "false"]
         if set_complete:
             changes = True
@@ -151,7 +161,7 @@ class Storage(storage.Storage):
 
         return changes or len(to_delete + to_uncomplete) > 0
 
-    def __get_category_list(self):
+    def __get_categories(self):
         return self.fetchall("SELECT * FROM categories ORDER BY category_order")
 
     def __change_category(self, id, category_id):
@@ -159,7 +169,7 @@ class Storage(storage.Storage):
         activity = self.fetchone("select name from activities where id = ?", (id, ))
         existing_activity = self.__get_activity_by_name(activity['name'], category_id)
 
-        if id == existing_activity['id']: # we are already there, go home
+        if existing_activity and id == existing_activity['id']: # we are already there, go home
             return False
 
         if existing_activity: #ooh, we have something here!
@@ -226,7 +236,7 @@ class Storage(storage.Storage):
 
 
 
-    def __get_activity_by_name(self, name, category_id = None, ressurect = True):
+    def __get_activity_by_name(self, name, category_id = None, resurrect = True):
         """get most recent, preferably not deleted activity by it's name"""
 
         if category_id:
@@ -253,11 +263,12 @@ class Storage(storage.Storage):
             res = self.fetchone(query, (_("Unsorted"), name, ))
 
         if res:
-            # if the activity was marked as deleted, ressurect on first call
+            res = dict(res)
+            res['deleted'] = res['deleted'] or False
+
+            # if the activity was marked as deleted, resurrect on first call
             # and put in the unsorted category
-            if res['deleted'] and not ressurect:
-                return None
-            elif res['deleted']:
+            if res['deleted'] and resurrect:
                 update = """
                             UPDATE activities
                                SET deleted = null, category_id = -1
@@ -269,7 +280,7 @@ class Storage(storage.Storage):
 
         return None
 
-    def __get_category_by_name(self, name):
+    def __get_category_id(self, name):
         """returns category by it's name"""
 
         query = """
@@ -327,14 +338,9 @@ class Storage(storage.Storage):
             grouped_facts.append(grouped_fact)
         return grouped_facts
 
-    def __get_last_activity(self):
-        facts = self.__get_todays_facts()
-        last_activity = None
-        if facts and facts[-1]["end_time"] == None:
-            last_activity = facts[-1]
-        return last_activity
 
     def __touch_fact(self, fact, end_time):
+        end_time = end_time or dt.datetime.now()
         # tasks under one minute do not count
         if end_time - fact['start_time'] < datetime.timedelta(minutes = 1):
             self.__remove_fact(fact['id'])
@@ -423,7 +429,7 @@ class Storage(storage.Storage):
                                    SET end_time = ?
                                  WHERE id = ?""", (start_time, fact["id"]))
                 fact_name = fact["name"]
-                new_fact = self.__add_fact(fact["name"],
+                new_fact_id = self.__add_fact(fact["name"],
                                            "", # will create tags in the next step
                                            end_time,
                                            fact["end_time"],
@@ -433,7 +439,7 @@ class Storage(storage.Storage):
                                      SELECT ?, tag_id
                                        FROM fact_tags
                                       WHERE fact_id = ?"""
-                self.execute(tag_update, (new_fact["id"], fact["id"])) #clone tags
+                self.execute(tag_update, (new_fact_id, fact["id"])) #clone tags
 
             #eliminate
             elif fact["end_time"] and \
@@ -456,13 +462,14 @@ class Storage(storage.Storage):
 
 
     def __add_fact(self, activity_name, tags, start_time = None,
-                     end_time = None, category_name = None, description = None):
+                   end_time = None, category_name = None,
+                   description = None, temporary = False):
 
         activity = stuff.parse_activity_input(activity_name)
 
         # make sure that we do have an activity name after parsing
         if not activity.activity_name:
-            return
+            return 0
 
         # explicitly stated takes precedence
         activity.description = description or activity.description
@@ -470,7 +477,7 @@ class Storage(storage.Storage):
         tags = [tag.strip() for tag in tags.split(",") if tag.strip()]  # split by comma
         tags = tags or activity.tags # explicitly stated tags take priority
 
-        tags = self.get_tag_ids(tags) #this will create any missing tags too
+        tags = self.GetTagIds(tags) #this will create any missing tags too
 
 
         if category_name:
@@ -480,34 +487,44 @@ class Storage(storage.Storage):
 
         start_time = activity.start_time or start_time or datetime.datetime.now()
 
-        if start_time > datetime.datetime.now():
-            return None #no facts in future, please
-
         start_time = start_time.replace(microsecond = 0)
         end_time = activity.end_time or end_time
         if end_time:
             end_time = end_time.replace(microsecond = 0)
 
+        now = datetime.datetime.now()
+        # if in future - roll back to past
+        if start_time > datetime.datetime.now():
+            start_time = dt.datetime.combine(now.date(),  start_time.time())
+            if start_time > now:
+                start_time -= dt.timedelta(days = 1)
+
+        if end_time and end_time > now:
+            end_time = dt.datetime.combine(now.date(),  end_time.time())
+            if end_time > now:
+                end_time -= dt.timedelta(days = 1)
+
+
         if not start_time or not activity.activity_name:  # sanity check
-            return
+            return 0
 
         # now check if maybe there is also a category
         category_id = None
         if activity.category_name:
-            category_id = self.__get_category_by_name(activity.category_name)
+            category_id = self.__get_category_id(activity.category_name)
             if not category_id:
                 category_id = self.__add_category(activity.category_name)
 
-        # try to find activity
+        # try to find activity, resurrect if not temporary
         activity_id = self.__get_activity_by_name(activity.activity_name,
-                                                  category_id)
+                                                  category_id,
+                                                  resurrect = not temporary)
         if not activity_id:
             activity_id = self.__add_activity(activity.activity_name,
-                                              category_id)
+                                              category_id, temporary)
         else:
             activity_id = activity_id['id']
 
-
         # if we are working on +/- current day - check the last_activity
         if (dt.datetime.now() - start_time <= dt.timedelta(days=1)):
 
@@ -523,7 +540,7 @@ class Storage(storage.Storage):
                 if previous["activity_id"] == activity_id \
                    and previous["tags"] == sorted([tag["name"] for tag in tags]) \
                    and previous["description"] == (description or ""):
-                    return previous
+                    return previous["id"]
 
                 # otherwise, if no description is added
                 # see if maybe it is too short to qualify as an activity
@@ -546,7 +563,7 @@ class Storage(storage.Storage):
                             """
                             self.execute(update, (before["id"],))
 
-                            return before
+                            return before["id"]
                 else:
                     # otherwise stop
                     update = """
@@ -710,66 +727,42 @@ class Storage(storage.Storage):
 
         return res
 
-    def __get_popular_categories(self):
-        """returns categories used in the specified interval"""
-        query = """
-                   SELECT coalesce(c.name, ?) as category, count(a.id) as popularity
-                     FROM facts a
-                LEFT JOIN activities b on a.activity_id = b.id
-                LEFT JOIN categories c on c.id = b.category_id
-                 GROUP BY b.category_id
-                 ORDER BY popularity desc
-        """
-        return self.fetchall(query, (_("Unsorted"), ))
-
     def __remove_fact(self, fact_id):
         statements = ["DELETE FROM fact_tags where fact_id = ?",
                       "DELETE FROM facts where id = ?"]
         self.execute(statements, [(fact_id,)] * 2)
 
-    def __get_activities(self, category_id = None):
+    def __get_category_activities(self, category_id):
         """returns list of activities, if category is specified, order by name
            otherwise - by activity_order"""
-        if category_id:
-            query = """
-                       SELECT a.*, b.name as category
-                         FROM activities a
-                    LEFT JOIN categories b on coalesce(b.id, -1) = a.category_id
-                        WHERE category_id = ?
-                          AND deleted is null
-            """
-
-            # unsorted entries we sort by name - others by ID
-            if category_id == -1:
-                query += "ORDER BY lower(a.name)"
-            else:
-                query += "ORDER BY a.activity_order"
-
-            activities = self.fetchall(query, (category_id, ))
+        query = """
+                   SELECT a.id, a.name, a.activity_order, a.category_id, b.name as category
+                     FROM activities a
+                LEFT JOIN categories b on coalesce(b.id, -1) = a.category_id
+                    WHERE category_id = ?
+                      AND deleted is null
+        """
 
+        # unsorted entries we sort by name - others by ID
+        if category_id == -1:
+            query += "ORDER BY lower(a.name)"
         else:
-            query = """
-                       SELECT a.*, b.name as category
-                         FROM activities a
-                    LEFT JOIN categories b on coalesce(b.id, -1) = a.category_id
-                        WHERE deleted IS NULL
-                     ORDER BY lower(a.name)
-            """
-            activities = self.fetchall(query)
+            query += "ORDER BY a.activity_order"
 
-        return activities
+        return self.fetchall(query, (category_id, ))
 
-    def __get_autocomplete_activities(self, search):
+
+    def __get_activities(self, search):
         """returns list of activities for autocomplete,
            activity names converted to lowercase"""
 
         query = """
-                   SELECT lower(a.name) AS name, b.name AS category
+                   SELECT a.name AS name, b.name AS category
                      FROM activities a
                 LEFT JOIN categories b ON coalesce(b.id, -1) = a.category_id
                 LEFT JOIN facts f ON a.id = f.activity_id
                     WHERE deleted IS NULL
-                      AND a.name LIKE ? ESCAPE '\\'
+                      AND lower(a.name) LIKE ? ESCAPE '\\'
                  GROUP BY a.id
                  ORDER BY max(f.start_time) DESC, lower(a.name)
                     LIMIT 50
@@ -808,7 +801,7 @@ class Storage(storage.Storage):
                       "update activities set activity_order = ? where id = ?"],
                       [(priority1, id2), (priority2, id1)])
 
-    def __add_activity(self, name, category_id = None):
+    def __add_activity(self, name, category_id = None, temporary = False):
         # first check that we don't have anything like that yet
         activity = self.__get_activity_by_name(name, category_id)
         if activity:
@@ -818,11 +811,16 @@ class Storage(storage.Storage):
         category_id = category_id or -1
         new_order = self.fetchone("select max(activity_order) + 1  from activities")[0] or 1
 
+        deleted = None
+        if temporary:
+            deleted = 1
+
+
         query = """
-                   INSERT INTO activities (name, category_id, activity_order)
-                        VALUES (?, ?, ?)
+                   INSERT INTO activities (name, category_id, activity_order, deleted)
+                        VALUES (?, ?, ?, ?)
         """
-        self.execute(query, (name, category_id, new_order))
+        self.execute(query, (name, category_id, new_order, deleted))
         return self.__last_insert_rowid()
 
     def __update_activity(self, id, name, category_id):
@@ -836,10 +834,8 @@ class Storage(storage.Storage):
 
     """ Here be dragons (lame connection/cursor wrappers) """
     def get_connection(self):
-        from configuration import runtime
         if self.con is None:
-            db_file = runtime.database_path
-            self.con = sqlite.connect(db_file, detect_types=sqlite.PARSE_DECLTYPES|sqlite.PARSE_COLNAMES)
+            self.con = sqlite.connect(self.db_path, detect_types=sqlite.PARSE_DECLTYPES|sqlite.PARSE_COLNAMES)
             self.con.row_factory = sqlite.Row
 
         return self.con
@@ -847,9 +843,6 @@ class Storage(storage.Storage):
     connection = property(get_connection, None)
 
     def fetchall(self, query, params = None):
-        from configuration import runtime
-        self.__setup()
-
         con = self.connection
         cur = con.cursor()
 
@@ -877,9 +870,6 @@ class Storage(storage.Storage):
         execute sql statement. optionally you can give multiple statements
         to save on cursor creation and closure
         """
-        from configuration import runtime
-        self.__setup()
-
         con = self.__con or self.connection
         cur = self.__cur or con.cursor()
 
@@ -896,7 +886,7 @@ class Storage(storage.Storage):
         if not self.__con:
             con.commit()
             cur.close()
-            runtime.register_modification()
+            self.register_modification()
 
 
     def start_transaction(self):
@@ -908,8 +898,7 @@ class Storage(storage.Storage):
         self.__con.commit()
         self.__cur.close()
         self.__con = None
-        from configuration import runtime
-        runtime.register_modification()
+        self.register_modification()
 
     def run_fixtures(self):
         self.start_transaction()
@@ -1071,7 +1060,7 @@ class Storage(storage.Storage):
             # previously deleted records are now unsorted ones
             # user will be able to mark them as deleted again, in which case
             # they won't appear in autocomplete, or in categories
-            # ressurection happens, when user enters the exact same name
+            # resurrection happens, when user enters the exact same name
             self.execute("""
                                CREATE TABLE activities_new (id integer primary key,
                                                             name varchar2(500),
