@@ -28,40 +28,12 @@ from pango import ELLIPSIZE_END
 
 from itertools import groupby
 import datetime as dt
+import calendar
 import time
 import re
-import gettext, locale
+import locale
 import os
 
-def setup_i18n():
-    #determine location of po files
-    try:
-        import defs
-    except:
-        defs = None
-
-
-    # to avoid confusion, we won't translate unless running installed
-    # reason for that is that bindtextdomain is expecting
-    # localedir/language/LC_MESSAGES/domain.mo format, but we have
-    # localedir/language.mo at it's best (after build)
-    # and there does not seem to be any way to run straight from sources
-    if defs:
-        locale_dir = os.path.realpath(os.path.join(defs.DATA_DIR, "locale"))
-
-        for module in (locale,):
-            module.bindtextdomain('hamster-applet', locale_dir)
-            module.textdomain('hamster-applet')
-
-            module.bind_textdomain_codeset('hamster-applet','utf8')
-
-        gettext.install("hamster-applet", locale_dir, unicode = True)
-
-    else:
-        gettext.install("hamster-applet-uninstalled")
-
-
-
 def format_duration(minutes, human = True):
     """formats duration in a human readable format.
     accepts either minutes or timedelta"""
@@ -95,6 +67,55 @@ def format_duration(minutes, human = True):
 
     return formatted_duration
 
+
+def format_range(start_date, end_date):
+    dates_dict = dateDict(start_date, "start_")
+    dates_dict.update(dateDict(end_date, "end_"))
+
+    if start_date == end_date:
+        # label of date range if looking on single day
+        # date format for overview label when only single day is visible
+        # Using python datetime formatting syntax. See:
+        # http://docs.python.org/library/time.html#time.strftime
+        title = start_date.strftime(_("%B %d, %Y"))
+    elif start_date.year != end_date.year:
+        # label of date range if start and end years don't match
+        # letter after prefixes (start_, end_) is the one of
+        # standard python date formatting ones- you can use all of them
+        # see http://docs.python.org/library/time.html#time.strftime
+        title = _(u"%(start_B)s %(start_d)s, %(start_Y)s – %(end_B)s %(end_d)s, %(end_Y)s") % dates_dict
+    elif start_date.month != end_date.month:
+        # label of date range if start and end month do not match
+        # letter after prefixes (start_, end_) is the one of
+        # standard python date formatting ones- you can use all of them
+        # see http://docs.python.org/library/time.html#time.strftime
+        title = _(u"%(start_B)s %(start_d)s – %(end_B)s %(end_d)s, %(end_Y)s") % dates_dict
+    else:
+        # label of date range for interval in same month
+        # letter after prefixes (start_, end_) is the one of
+        # standard python date formatting ones- you can use all of them
+        # see http://docs.python.org/library/time.html#time.strftime
+        title = _(u"%(start_B)s %(start_d)s – %(end_d)s, %(end_Y)s") % dates_dict
+
+    return title
+
+
+
+def week(view_date):
+    # aligns start and end date to week
+    start_date = view_date - dt.timedelta(view_date.weekday() + 1)
+    start_date = start_date + dt.timedelta(locale_first_weekday())
+    end_date = start_date + dt.timedelta(6)
+    return start_date, end_date
+
+def month(view_date):
+    # aligns start and end date to month
+    start_date = view_date - dt.timedelta(view_date.day - 1) #set to beginning of month
+    first_weekday, days_in_month = calendar.monthrange(view_date.year, view_date.month)
+    end_date = start_date + dt.timedelta(days_in_month - 1)
+    return start_date, end_date
+
+
 def duration_minutes(duration):
     """returns minutes from duration, otherwise we keep bashing in same math"""
     return duration.seconds / 60 + duration.days * 24 * 60
@@ -143,33 +164,6 @@ def locale_first_weekday():
 
     return first_weekday
 
-class CategoryCell(gtk.CellRendererText):
-    def __init__(self):
-        gtk.CellRendererText.__init__(self)
-        self.set_property('alignment', pango.ALIGN_RIGHT)
-
-        insensitive_color = gtk.Label().style.fg[gtk.STATE_INSENSITIVE]
-        self.set_property('foreground-gdk', insensitive_color)
-        self.set_property('scale', pango.SCALE_SMALL)
-        self.set_property('yalign', 0.0)
-
-
-insensitive_color = gtk.Label().style.fg[gtk.STATE_INSENSITIVE].to_string()
-def format_activity(name, category, description, pad_description = False):
-    "returns pango markup for activity with category and description"
-    text = name
-    if category and category != _("Unsorted"):
-        text += """ - <span color="%s" size="x-small">%s</span>""" % (insensitive_color, category)
-
-    if description:
-        text+= "\n"
-        if pad_description:
-            text += " " * 23
-
-        text += """<span style="italic" size="small">%s</span>""" % description
-
-    return text
-
 
 def totals(iter, keyfunc, sumfunc):
     """groups items by field described in keyfunc and counts totals using value
@@ -184,8 +178,6 @@ def totals(iter, keyfunc, sumfunc):
     return res
 
 
-
-
 def dateDict(date, prefix = ""):
     """converts date into dictionary, having prefix for all the keys"""
     res = {}
